import numpy as np
import math
from numpy import tensordot


class Linear():
    # DO NOT DELETE
    def __init__(self, in_feature, out_feature):
        self.in_feature = in_feature
        self.out_feature = out_feature

        self.W = np.random.randn(out_feature, in_feature)
        self.b = np.zeros(out_feature)
        
        self.dW = np.zeros(self.W.shape)
        self.db = np.zeros(self.b.shape)

    def __call__(self, x):
        return self.forward(x)

    def forward(self, x):
        self.x = x
        self.out = x.dot(self.W.T) + self.b
        return self.out

    def backward(self, delta):
        self.db = delta
        self.dW = np.dot(self.x.T, delta)
        dx = np.dot(delta, self.W.T)
        return dx

        

class Conv1D():
    def __init__(self, in_channel, out_channel, 
                 kernel_size, stride):

        self.in_channel = in_channel
        self.out_channel = out_channel
        self.kernel_size = kernel_size
        self.stride = stride

        self.W = np.random.randn(out_channel, in_channel, kernel_size)
        self.b = np.zeros(out_channel)

    def __call__(self, x):
        return self.forward(x)

    def forward(self, x):

        '''
        Parameters
        ----------

        x: 
            -Is a numpy ndarray (i.e. type(x): numpy.ndarray)

            -Is the input to the 1-D convolutional layer

            -Has a shape of (batch_size, in_channel, in_width) where:

                -batch_size -> number of 1-D input sequences in a single batch

                -in_channel -> number of channels in the input sequence

                -in_width -> width of the input sequence

        Returns
        -------

        output: 

            -Is a numpy ndarray(i.e. type(output): numpy.ndarray)

            -Is the output of the 1-D convolution

            -Has a shape of (batch_size, out_channel, out_width) where:

                -out_channel -> number of channels in the output of the convolution

                -out_width -> width of the output sequence

        Note: Please refer to homework notes(link is provided in the writeup) for HW2 
        for a detailed explnation of the parameters(e.g. out_channel, in_width, etc.) and convolution.

        '''

        ## Your codes here
        self.batch, __ , self.width = x.shape
        
        assert __ == self.in_channel, 'Expected the inputs to have {} channels'.format(self.in_channel)

        raise NotImplemented

