import torch
import torch.nn as nn
import numpy as np
from  torch.autograd import Variable
import multiprocessing as mtp
import traceback


np.random.seed(11785)


def test_cnn_correctness_once(idx):
    from layers import Conv1D

    scores_dict = [0]

    ############################################################################################
    #############################   Initialize hyperparameters    ##############################
    ############################################################################################
    rint = np.random.randint
    norm = np.linalg.norm
    in_c, out_c = rint(5,15), rint(5,15)
    kernel, stride =  rint(1,10), rint(1,10)
    batch, width = rint(1,4), rint(20,300)



    def info():
        print('\nTesting model:')
        print('    in_channel: {}, out_channel: {},'.format(in_c,out_c))
        print('    kernel size: {}, stride: {},'.format(kernel,stride))
        print('    batch size: {}, input size: {}.'.format(batch,width))


    ##############################################################################################
    ##########    Initialize the CNN layer and copy parameters to a PyTorch CNN layer   ##########
    ##############################################################################################
    try:
        net = Conv1D(in_c, out_c, kernel, stride)
    except:
        info()
        print('Failed to pass parameters to your Conv1D function!')
        return scores_dict

    model = nn.Conv1d(in_c, out_c, kernel, stride)
    model.weight = nn.Parameter(torch.tensor(net.W))
    model.bias = nn.Parameter(torch.tensor(net.b))


    #############################################################################################
    #########################    Get the correct results from PyTorch   #########################
    #############################################################################################
    x = np.random.randn(batch, in_c, width)
    x1 = Variable(torch.tensor(x),requires_grad=True)
    y1 = model(x1)
    b, c, w = y1.shape
    delta = np.random.randn(b,c,w)


    #############################################################################################
    ##########################    Get your forward results and compare ##########################
    #############################################################################################
    y = net(x)
    assert y.shape == y1.shape
    if not(y.shape == y1.shape): print("FAILURE")


    forward_res = y - y1.detach().numpy()
    forward_res_norm = abs(forward_res).max()


    if forward_res_norm < 1e-12:
        scores_dict[0] =  1
    else:
        info()
        print('Fail to return correct forward values')
        return scores_dict


    return scores_dict


def test_cnn_correctness():
    scores = []
    worker = min(mtp.cpu_count(),4)
    p = mtp.Pool(worker)
    
    for __ in range(15):
        scores_dict = test_cnn_correctness_once(__) 
        scores.append(scores_dict)
        if min(scores_dict) != 1:
            break
    scores = np.array(scores).min(0)
    return list(scores)




if __name__ == '__main__':
    a = test_cnn_correctness()
    
    print('Conv1D Forward:', 'PASS' if a[0] == 1 else 'FAIL')

  



    





